@@ -42,6 +42,8 @@
 #include "error.h"
 #include "external/date.h"
 
+namespace gdalcubes {
+
 enum class datetime_unit {
     SECOND = 0,
     MINUTE = 1,
@@ -562,4 +564,6 @@ class datetime {
     }
 };
 
+}  // namespace gdalcubes
+
 #endif  //DATETIME_H
