@@ -32,7 +32,8 @@
 #include "config.h"
 #include "cube.h"
 #include "dummy.h"
-#include "filter_predicate.h"
+#include "fill_time.h"
+#include "filter_pixel.h"
 #include "image_collection_cube.h"
 #include "join_bands.h"
 #include "progress.h"
@@ -41,6 +42,8 @@
 #include "reduce_time.h"
 #include "select_bands.h"
 #include "stream.h"
+#include "stream_apply_pixel.h"
+#include "stream_reduce_time.h"
 #include "swarm.h"
 #include "utils.h"
 #include "window_time.h"
